@@ -10,33 +10,15 @@
 AccountTrustMessageUriGenerator::AccountTrustMessageUriGenerator(QObject *parent)
     : TrustMessageUriGenerator(parent)
 {
-    connect(this, &AccountTrustMessageUriGenerator::jidChanged, this, &AccountTrustMessageUriGenerator::setUp);
-}
-
-void AccountTrustMessageUriGenerator::setUp()
-{
-    connect(EncryptionController::instance(),
-            &EncryptionController::keysChanged,
-            this,
-            &AccountTrustMessageUriGenerator::handleKeysChanged,
-            Qt::UniqueConnection);
-    updateKeys();
-}
-
-void AccountTrustMessageUriGenerator::handleKeysChanged(const QString &accountJid, const QList<QString> &jids)
-{
-    if (jid() == accountJid && jids.contains(jid())) {
-        updateKeys();
-    }
 }
 
 void AccountTrustMessageUriGenerator::updateKeys()
 {
-    EncryptionController::instance()->ownKey(jid()).then(this, [this](QString &&key) {
+    encryptionController()->ownKey().then(this, [this](QString &&key) {
         QList<QString> authenticatedKeys = {key};
 
-        EncryptionController::instance()
-            ->keys(jid(), {jid()}, QXmpp::TrustLevel::ManuallyDistrusted | QXmpp::TrustLevel::Authenticated)
+        encryptionController()
+            ->keys({jid()}, QXmpp::TrustLevel::ManuallyDistrusted | QXmpp::TrustLevel::Authenticated)
             .then(this, [this, authenticatedKeys](QHash<QString, QHash<QString, QXmpp::TrustLevel>> &&keys) mutable {
                 const auto keyIds = keys.value(jid());
                 QList<QString> distrustedKeys;
