@@ -8,16 +8,23 @@
 #include <QXmppUri.h>
 
 GroupChatQrCodeGenerator::GroupChatQrCodeGenerator(QObject *parent)
-    : AbstractQrCodeGenerator(parent)
+    : QrCodeGenerator(parent)
 {
-    connect(this, &GroupChatQrCodeGenerator::jidChanged, this, &GroupChatQrCodeGenerator::updateText);
+}
+
+void GroupChatQrCodeGenerator::setJid(const QString &jid)
+{
+    if (m_jid != jid) {
+        m_jid = jid;
+        updateText();
+    }
 }
 
 void GroupChatQrCodeGenerator::updateText()
 {
     QXmppUri uri;
 
-    uri.setJid(jid());
+    uri.setJid(m_jid);
     uri.setQuery(QXmpp::Uri::Join());
 
     setText(uri.toString());
