@@ -29,8 +29,10 @@
 #include "KaidanCoreLog.h"
 #include "SystemUtils.h"
 
+const auto THUMBNAIL_FORMAT = "PNG";
+
 const auto IMAGE_FILE_EXTENSION = QStringLiteral("jpg");
-const auto AUDIO_FILE_EXTENSION = QStringLiteral("mp3");
+const auto AUDIO_FILE_EXTENSION = QStringLiteral("m4a");
 const auto VIDEO_FILE_EXTENSION = QStringLiteral("mp4");
 
 const auto FILE_PATH = QStringLiteral("%1/%2.%3");
@@ -525,11 +527,46 @@ Enums::MessageType MediaUtils::messageType(const QMimeType &mimeType)
     return Enums::MessageType::MessageFile;
 }
 
+bool MediaUtils::hasAlphaChannel(const QVariant &source)
+{
+    const auto determineAlphaChannel = [](const auto &image) {
+        return image.hasAlphaChannel();
+    };
+
+    const auto determineIconAlphaChannel = [determineAlphaChannel](const QIcon &icon) {
+        return determineAlphaChannel(icon.pixmap(64, 64));
+    };
+
+    switch (source.typeId()) {
+    case QMetaType::Type::QImage:
+        return determineAlphaChannel(source.value<QImage>());
+    case QMetaType::Type::QPixmap:
+        return determineAlphaChannel(source.value<QPixmap>());
+    case QMetaType::Type::QIcon:
+        return determineIconAlphaChannel(source.value<QIcon>());
+    case QMetaType::Type::QUrl:
+        qCDebug(KAIDAN_CORE_LOG, "Alpha channel of source specified by QUrl cannot be determined");
+        break;
+    case QMetaType::Type::QString: {
+        if (const auto image = QImage(source.toString()); !image.isNull()) {
+            return determineAlphaChannel(image);
+        } else if (const auto icon = QIcon::fromTheme(source.toString()); !icon.isNull()) {
+            return determineIconAlphaChannel(icon);
+        }
+
+        qCDebug(KAIDAN_CORE_LOG, "Alpha channel could not be determined because no valid image data could be loaded from QString");
+        break;
+    }
+    }
+
+    return false;
+}
+
 QByteArray MediaUtils::encodeImageThumbnail(const QPixmap &pixmap)
 {
     QByteArray output;
     QBuffer buffer(&output);
-    pixmap.save(&buffer, "PNG");
+    pixmap.save(&buffer, THUMBNAIL_FORMAT);
     return output;
 }
 
@@ -537,7 +574,7 @@ QByteArray MediaUtils::encodeImageThumbnail(const QImage &image)
 {
     QByteArray output;
     QBuffer buffer(&output);
-    image.save(&buffer, "PNG");
+    image.save(&buffer, THUMBNAIL_FORMAT);
     return output;
 }
 
@@ -566,7 +603,8 @@ QFuture<std::shared_ptr<QXmppFileSharingManager::MetadataGeneratorResult>> Media
     connect(job, &KIO::PreviewJob::gotPreview, job, [=, f = std::move(f)](const KFileItem &, const QPixmap &image) mutable {
         QByteArray thumbnailData;
         QBuffer thumbnailBuffer(&thumbnailData);
-        image.save(&thumbnailBuffer, "JPG", JPEG_EXPORT_QUALITY);
+
+        image.save(&thumbnailBuffer, THUMBNAIL_FORMAT, THUMBNAIL_QUALITY);
 
         if (const QImageReader reader(file->fileName()); reader.canRead()) {
             if (const QSize size = reader.size(); !size.isEmpty()) {
