@@ -74,10 +74,16 @@ QXmppFileShare File::toQXmpp() const
     metadata.setLastModified(lastModified);
     metadata.setMediaType(mimeType);
     metadata.setSize(size);
+    metadata.setWidth(width);
+    metadata.setHeight(height);
+
+    const auto thumbnailImage = QImage::fromData(thumbnail);
 
     QXmppThumbnail thumb;
     thumb.setMediaType(QMimeDatabase().mimeTypeForData(thumbnail));
     thumb.setUri(QXmppBitsOfBinaryData::fromByteArray(thumbnail).cid().toCidUrl());
+    thumb.setWidth(thumbnailImage.width());
+    thumb.setHeight(thumbnailImage.height());
     metadata.setThumbnails({thumb});
 
     QXmppFileShare fs;
@@ -233,7 +239,7 @@ QXmppMessage Message::toQXmpp() const
         // "toJid" and "toGroupChatParticipantId" are empty if the reply is to an own message.
         if (toJid.isEmpty() && toGroupChatParticipantId.isEmpty()) {
             if (isGroupChatMessage()) {
-                qxmppReply.to = RosterModel::instance()->findItem(chatJid)->groupChatParticipantId;
+                qxmppReply.to = RosterModel::instance()->item(accountJid, chatJid)->groupChatParticipantId;
             } else {
                 qxmppReply.to = accountJid;
             }
