@@ -58,10 +58,10 @@ std::optional<T> variantToOptional(QVariant value)
 
 MessageDb *MessageDb::s_instance = nullptr;
 
-MessageDb::MessageDb(Database *db, QObject *parent)
-    : DatabaseComponent(db, parent)
+MessageDb::MessageDb(QObject *parent)
+    : DatabaseComponent(parent)
 {
-    Q_ASSERT(!MessageDb::s_instance);
+    Q_ASSERT(!s_instance);
     s_instance = this;
 
     _fetchLatestFileId();
@@ -122,6 +122,7 @@ QList<Message> MessageDb::_fetchMessagesFromQuery(QSqlQuery &query)
     int idxGroupChatInvitationJid = rec.indexOf(QStringLiteral("groupChatInvitationJid"));
     int idxGroupChatToken = rec.indexOf(QStringLiteral("groupChatToken"));
     int idxErrorText = rec.indexOf(QStringLiteral("errorText"));
+    int idxMarked = rec.indexOf(QStringLiteral("marked"));
     int idxRemoved = rec.indexOf(QStringLiteral("removed"));
 
     reserve(messages, query);
@@ -177,6 +178,7 @@ QList<Message> MessageDb::_fetchMessagesFromQuery(QSqlQuery &query)
             };
         }
         msg.errorText = query.value(idxErrorText).toString();
+        msg.marked = query.value(idxMarked).toBool();
         msg.removed = query.value(idxRemoved).toBool();
 
         messages << std::move(msg);
@@ -267,6 +269,9 @@ QSqlRecord MessageDb::createUpdateRecord(const Message &oldMsg, const Message &n
     if (oldMsg.removed != newMsg.removed) {
         rec.append(createSqlField(QStringLiteral("removed"), newMsg.removed));
     }
+    if (oldMsg.marked != newMsg.marked) {
+        rec.append(createSqlField(QStringLiteral("marked"), newMsg.marked));
+    }
 
     return rec;
 }
@@ -456,7 +461,7 @@ MessageDb::fetchMessagesUntilId(const QString &accountJid, const QString &chatJi
                       {u":accountJid", accountJid},
                       {u":chatJid", chatJid},
                       {u":index", index},
-                      {u":limit", messagesUntilFoundMessageCount + DB_QUERY_LIMIT_MESSAGES},
+                      {u":limit", messagesUntilFoundMessageCount ? messagesUntilFoundMessageCount + DB_QUERY_LIMIT_MESSAGES * 2 : DB_QUERY_LIMIT_MESSAGES},
                   });
 
         MessageResult result{_fetchMessagesFromQuery(query),
@@ -610,60 +615,92 @@ bool MessageDb::_hasMessage(const QString &accountJid, const QString &chatJid, c
     return query.first();
 }
 
-QFuture<int> MessageDb::messageCount(const QString &accountJid, const QString &chatJid, const QString &messageIdBegin, const QString &messageIdEnd)
+QFuture<int> MessageDb::latestContactMessageCount(const QString &accountJid, const QString &chatJid, const QString &messageIdBegin)
 {
-    return run([=, this]() {
-        auto query = createQuery();
-        execQuery(query,
-                  // The double round brackets are needed to evaluate the datetime.
-                  QStringLiteral(R"(
-				SELECT COUNT(*)
-				FROM chatMessages DESC
-				WHERE
-					accountJid = :accountJid AND chatJid = :chatJid AND
-					datetime(timestamp) BETWEEN
-						datetime((
-							SELECT timestamp
-							FROM chatMessages DESC
-							WHERE accountJid = :accountJid AND chatJid = :chatJid AND id = :messageIdBegin
-							LIMIT 1
-						)) AND
-						datetime((
-							SELECT timestamp
-							FROM chatMessages DESC
-							WHERE accountJid = :accountJid AND chatJid = :chatJid AND id = :messageIdEnd
-							LIMIT 1
-						))
-			)"),
-                  {
-                      {u":accountJid", accountJid},
-                      {u":chatJid", chatJid},
-                      {u":messageIdBegin", messageIdBegin},
-                      {u":messageIdEnd", messageIdEnd},
-                  });
-
-        if (query.first()) {
-            return query.value(0).toInt();
-        }
-
-        return 0;
+    return run([this, accountJid, chatJid, messageIdBegin] {
+        return _latestContactMessageCount(accountJid, chatJid, messageIdBegin);
     });
 }
 
-QFuture<void>
-MessageDb::addOrUpdateMessage(const Message &message, MessageOrigin origin, const QString &originId, const std::function<void(Message &)> &updateMessage)
+int MessageDb::_latestContactMessageCount(const QString &accountJid, const QString &chatJid, const QString &messageIdBegin)
 {
-    Q_ASSERT(message.deliveryState != DeliveryState::Draft);
+    auto query = createQuery();
+    execQuery(query,
+              QStringLiteral(R"(
+				SELECT COUNT(*)
+				FROM chatMessages
+				WHERE
+                    accountJid = :accountJid AND chatJid = :chatJid AND isOwn = 0 AND
+                    (
+                        :messageIdBegin IS NULL OR
+                        (
+                            timestamp >
+                            (
+                                SELECT timestamp
+                                FROM chatMessages
+                                WHERE accountJid = :accountJid AND chatJid = :chatJid AND id = :messageIdBegin
+                                LIMIT 1
+                            )
+                        )
+                    )
+			)"),
+              {
+                  {u":accountJid", accountJid},
+                  {u":chatJid", chatJid},
+                  {u":messageIdBegin", messageIdBegin},
+              });
 
-    return run([this, message, origin, originId, updateMessage]() {
-        if (_checkMessageExists(message)) {
-            _updateMessage(originId, updateMessage);
-        } else {
-            _addMessage(message, origin);
-        }
+    query.first();
+    return query.value(0).toInt();
+}
+
+QFuture<int> MessageDb::markedMessageCount(const QString &accountJid, const QString &chatJid)
+{
+    return run([this, accountJid, chatJid] {
+        return _markedMessageCount(accountJid, chatJid);
     });
 }
 
+int MessageDb::_markedMessageCount(const QString &accountJid, const QString &chatJid)
+{
+    auto query = createQuery();
+    execQuery(query,
+              QStringLiteral(R"(
+				SELECT COUNT(*)
+				FROM chatMessages
+				WHERE
+                    accountJid = :accountJid AND chatJid = :chatJid AND marked = 1
+			)"),
+              {
+                  {u":accountJid", accountJid},
+                  {u":chatJid", chatJid},
+              });
+
+    query.first();
+    return query.value(0).toInt();
+}
+
+bool MessageDb::_checkMoreRecentMessageExists(const QString &accountJid, const QString &chatJid, const QDateTime &timestamp, int offset)
+{
+    auto query = createQuery();
+    execQuery(query,
+              QStringLiteral(R"(
+			SELECT COUNT(*)
+			FROM chatMessages
+			WHERE
+                accountJid = :accountJid AND chatJid = :chatJid AND
+                timestamp >= :timestamp
+		)"),
+              {
+                  {u":accountJid", accountJid},
+                  {u":chatJid", chatJid},
+                  {u":timestamp", timestamp.toString(Qt::ISODateWithMs)},
+              });
+
+    query.first();
+    return query.value(0).toInt() > offset;
+}
+
 QFuture<void> MessageDb::addMessage(const Message &message, MessageOrigin origin)
 {
     Q_ASSERT(message.deliveryState != DeliveryState::Draft);
@@ -673,7 +710,40 @@ QFuture<void> MessageDb::addMessage(const Message &message, MessageOrigin origin
     });
 }
 
-QFuture<void> MessageDb::removeAllMessagesFromAccount(const QString &accountJid)
+void MessageDb::_addMessage(Message message, MessageOrigin origin)
+{
+    _fetchGroupChatUser(message);
+    _fetchTrustLevel(message);
+    _fetchReply(message);
+
+    Q_EMIT messageAdded(message, origin);
+
+    _addMessage(message);
+    _setFiles(message.files);
+}
+
+QFuture<void> MessageDb::addOrUpdateMessage(const Message &message, MessageOrigin origin, const std::function<void(Message &)> &updateMessage)
+{
+    Q_ASSERT(message.deliveryState != DeliveryState::Draft);
+
+    return run([this, message, origin, updateMessage]() {
+        if (_checkMessageExists(message)) {
+            _updateMessage(message.accountJid, message.chatJid, message.originId, updateMessage);
+        } else {
+            _addMessage(message, origin);
+        }
+    });
+}
+
+QFuture<void>
+MessageDb::updateMessage(const QString &accountJid, const QString &chatJid, const QString &messageId, const std::function<void(Message &)> &updateMsg)
+{
+    return run([this, accountJid, chatJid, messageId, updateMsg]() {
+        _updateMessage(accountJid, chatJid, messageId, updateMsg);
+    });
+}
+
+QFuture<void> MessageDb::removeMessages(const QString &accountJid)
 {
     return run([this, accountJid]() {
         auto query = createQuery();
@@ -708,12 +778,10 @@ QFuture<void> MessageDb::removeAllMessagesFromAccount(const QString &accountJid)
                   {
                       {u":accountJid", accountJid},
                   });
-
-        allMessagesRemovedFromAccount(accountJid);
     });
 }
 
-QFuture<void> MessageDb::removeAllMessagesFromChat(const QString &accountJid, const QString &chatJid)
+QFuture<void> MessageDb::removeMessages(const QString &accountJid, const QString &chatJid)
 {
     return run([this, accountJid, chatJid]() {
         auto query = createQuery();
@@ -751,7 +819,7 @@ QFuture<void> MessageDb::removeAllMessagesFromChat(const QString &accountJid, co
                       {u":chatJid", chatJid},
                   });
 
-        allMessagesRemovedFromChat(accountJid, chatJid);
+        messagesRemoved(accountJid, chatJid);
     });
 }
 
@@ -819,13 +887,6 @@ QFuture<void> MessageDb::removeMessage(const QString &accountJid, const QString
     });
 }
 
-QFuture<void> MessageDb::updateMessage(const QString &id, const std::function<void(Message &)> &updateMsg)
-{
-    return run([this, id, updateMsg]() {
-        _updateMessage(id, updateMsg);
-    });
-}
-
 QFuture<void> MessageDb::attachFileSources(const QString &accountJid,
                                            const QString &chatJid,
                                            const QString &messageId,
@@ -891,12 +952,8 @@ QFuture<void> MessageDb::addDraftMessage(const Message &msg)
     return run([this, msg]() {
         Q_ASSERT(msg.deliveryState == DeliveryState::Draft);
 
-        const auto addMessage = [this](const Message &message) {
-            _addMessage(message);
-            Q_EMIT draftMessageAdded(message);
-        };
-
-        addMessage(msg);
+        _addMessage(msg);
+        Q_EMIT draftMessageAdded(msg);
     });
 }
 
@@ -949,18 +1006,6 @@ QFuture<void> MessageDb::removeDraftMessage(const QString &accountJid, const QSt
     });
 }
 
-void MessageDb::_addMessage(Message message, MessageOrigin origin)
-{
-    _fetchGroupChatUser(message);
-    _fetchTrustLevel(message);
-    _fetchReply(message);
-
-    Q_EMIT messageAdded(message, origin);
-
-    _addMessage(message);
-    _setFiles(message.files);
-}
-
 void MessageDb::_addMessage(const Message &message)
 {
     SqlUtils::QueryBindValues values = {
@@ -981,6 +1026,7 @@ void MessageDb::_addMessage(const Message &message)
         {u"spoilerHint", message.spoilerHint},
         {u"fileGroupId", optionalToVariant(message.fileGroupId)},
         {u"errorText", message.errorText},
+        {u"marked", message.marked},
         {u"removed", message.removed},
     };
 
@@ -1006,19 +1052,24 @@ void MessageDb::_addMessage(const Message &message)
     insertBinary(QString::fromLatin1(DB_TABLE_MESSAGES), values);
 }
 
-void MessageDb::_updateMessage(const QString &id, const std::function<void(Message &)> &updateMsg)
+void MessageDb::_updateMessage(const QString &accountJid, const QString &chatJid, const QString &messageId, const std::function<void(Message &)> &updateMsg)
 {
     // load current message item from db
     auto query = createQuery();
     execQuery(query,
               QStringLiteral(R"(
-			SELECT *
-			FROM chatMessages
-			WHERE replaceId = :id OR stanzaId = :id OR originId = :id OR id = :id
-			LIMIT 1
-		)"),
+                SELECT *
+                FROM chatMessages
+                WHERE
+                    accountJid = :accountJid AND
+                    chatJid = :chatJid AND
+                    (replaceId = :id OR stanzaId = :id OR originId = :id OR id = :id)
+                LIMIT 1
+              )"),
               {
-                  {u":id", id},
+                  {u":accountJid", accountJid},
+                  {u":chatJid", chatJid},
+                  {u":id", messageId},
               });
 
     auto msgs = _fetchMessagesFromQuery(query);
@@ -1113,7 +1164,12 @@ void MessageDb::_updateMessage(const QString &id, const std::function<void(Messa
                 // Create an SQL record containing only the differences.
                 execQuery(query,
                           driver.sqlStatement(QSqlDriver::UpdateStatement, QStringLiteral(DB_TABLE_MESSAGES), rec, false)
-                              + simpleWhereStatement(&driver, QStringLiteral("id"), oldMessage.id));
+                              + simpleWhereStatement(&driver,
+                                                     {
+                                                         {QStringLiteral("accountJid"), oldMessage.accountJid},
+                                                         {QStringLiteral("chatJid"), oldMessage.chatJid},
+                                                         {QStringLiteral("id"), oldMessage.id},
+                                                     }));
             }
 
             // remove old files
@@ -1655,7 +1711,7 @@ void MessageDb::_fetchTrustLevel(Message &message)
             // The message is sent from this device.
             message.preciseTrustLevel = QXmpp::TrustLevel::Authenticated;
         } else {
-            message.preciseTrustLevel = TrustDb::instance()->_trustLevel(XMLNS_OMEMO_2, message.senderJid(), senderKey);
+            message.preciseTrustLevel = TrustDb::_trustLevel(database(), message.accountJid, XMLNS_OMEMO_2, message.senderJid(), senderKey);
         }
     }
 }
@@ -1763,11 +1819,8 @@ bool MessageDb::_checkMessageExists(const Message &message)
     auto query = createQuery();
     execQuery(query, querySql, bindValues);
 
-    int count = 0;
-    if (query.next()) {
-        count = query.value(0).toInt();
-    }
-    return count > 0;
+    query.first();
+    return query.value(0).toInt() > 0;
 }
 
 QFuture<QList<Message>> MessageDb::fetchPendingMessages(const QString &accountJid)
@@ -1815,7 +1868,7 @@ QFuture<QMap<QString, QMap<QString, MessageReactionSender>>> MessageDb::fetchPen
                       {u":deliveryState3", int(MessageReactionDeliveryState::PendingRemovalAfterDelivered)},
                   });
 
-        // IDs of chats mapped to IDs of messages mapped to MessageReactionSender
+        // JIDs of chats mapped to IDs of messages mapped to MessageReactionSender
         QMap<QString, QMap<QString, MessageReactionSender>> reactions;
 
         // Iterate over all IDs of messages with pending reactions.
