@@ -15,7 +15,13 @@
 // Kaidan
 #include "Message.h"
 
-class Kaidan;
+class AccountSettings;
+class AtmController;
+class ChatController;
+class Connection;
+class EncryptionController;
+class MessageController;
+class NotificationController;
 
 struct DisplayedMessageReaction {
     Q_GADGET
@@ -85,14 +91,19 @@ public:
         OwnReactionsFailed,
         GroupChatInvitationJid,
         GeoCoordinate,
+        Marked,
         ErrorText,
     };
     Q_ENUM(MessageRoles)
 
-    static MessageModel *instance();
-
-    explicit MessageModel(QObject *parent = nullptr);
-    ~MessageModel() override;
+    MessageModel(AccountSettings *accountSettings,
+                 Connection *connection,
+                 AtmController *atmController,
+                 ChatController *chatController,
+                 EncryptionController *encryptionController,
+                 MessageController *messageController,
+                 NotificationController *notificationController,
+                 QObject *parent = nullptr);
 
     [[nodiscard]] int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     [[nodiscard]] QHash<int, QByteArray> roleNames() const override;
@@ -104,7 +115,7 @@ public:
 
     Q_INVOKABLE void handleMessageRead(int readMessageIndex);
     Q_INVOKABLE int firstUnreadContactMessageIndex() const;
-    Q_INVOKABLE void markMessageAsFirstUnread(int index);
+    Q_INVOKABLE void setMessageMarked(int index, bool marked);
 
     /**
      * Adds a message reaction or undoes its pending/failed removal.
@@ -201,10 +212,10 @@ private:
     void handleMessagesFetched(const QList<Message> &m_messages);
     void handleMamBacklogRetrieved(bool complete);
 
-    void handleMessage(Message msg, MessageOrigin origin);
+    void handleMessage(Message msg, MessageOrigin);
     void handleMessageUpdated(Message message);
 
-    void handleDevicesChanged(const QString &accountJid, QList<QString> jids);
+    void handleDevicesChanged(QList<QString> jids);
 
     void addMessage(const Message &msg);
     void insertMessage(int i, const Message &msg);
@@ -224,13 +235,6 @@ private:
 
     void emitMessagesUpdated(const QList<QString> &messageIds, MessageRoles role);
 
-    /**
-     * Shows a notification for the message when needed
-     *
-     * @param message message for which a notification might be shown
-     */
-    void showMessageNotification(const Message &message, MessageOrigin origin);
-
     /**
      * Undoes a pending or failed removal of a message reaction.
      *
@@ -278,15 +282,20 @@ private:
     QDate searchNextDate(int messageStartIndex) const;
 
     QString formatDate(QDate localDate) const;
-    QString determineGroupChatSenderName(const Message &message) const;
     QString determineReplyToName(const Message::Reply &reply) const;
 
+    AccountSettings *const m_accountSettings;
+    Connection *const m_connection;
+
+    AtmController *const m_atmController;
+    ChatController *const m_chatController;
+    MessageController *const m_messageController;
+    NotificationController *const m_notificationController;
+
     QList<Message> m_messages;
     QString m_lastReadOwnMessageId;
     int m_firstUnreadContactMessageIndex = -1;
     bool m_fetchedAllFromDb = false;
     bool m_fetchedAllFromMam = false;
     bool m_mamLoading = false;
-
-    static MessageModel *s_instance;
 };
