@@ -13,6 +13,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 // Qt
+#include <QApplication>
 #include <QCommandLineOption>
 #include <QCommandLineParser>
 #include <QDir>
@@ -22,8 +23,10 @@
 #include <QLibraryInfo>
 #include <QLocale>
 #include <QQmlApplicationEngine>
+#include <QRegularExpression>
 #include <QTranslator>
 #include <QWindow>
+#include <QtLogging>
 #include <qqml.h>
 #ifdef Q_OS_ANDROID
 #include <QtAndroid>
@@ -56,13 +59,13 @@
 #include <QXmppVersionIq.h>
 // Kaidan
 #include "Account.h"
-#include "AccountManager.h"
-#include "AccountMigrationManager.h"
+#include "AccountController.h"
+#include "AccountMigrationController.h"
 #include "AccountQrCodeGenerator.h"
-#include "AtmManager.h"
+#include "AtmController.h"
 #include "AuthenticatableEncryptionKeyModel.h"
 #include "AuthenticatedEncryptionKeyModel.h"
-#include "AvatarFileStorage.h"
+#include "AvatarCache.h"
 #include "BitsOfBinaryImageProvider.h"
 #include "Blocking.h"
 #include "ChatController.h"
@@ -83,6 +86,7 @@
 #include "FileSharingController.h"
 #include "GlobalsGen.h"
 #include "GroupChatController.h"
+#include "GroupChatInviteeFilterModel.h"
 #include "GroupChatQrCodeGenerator.h"
 #include "GroupChatUser.h"
 #include "GroupChatUserFilterModel.h"
@@ -91,9 +95,9 @@
 #include "GuiStyle.h"
 #include "HostCompletionModel.h"
 #include "HostCompletionProxyModel.h"
-#include "Kaidan.h"
 #include "KaidanLog.h"
 #include "LoginQrCodeGenerator.h"
+#include "MainController.h"
 #include "MediaUtils.h"
 #include "Message.h"
 #include "MessageComposition.h"
@@ -103,21 +107,21 @@
 #include "ProviderListModel.h"
 #include "PublicGroupChatModel.h"
 #include "PublicGroupChatProxyModel.h"
-#include "PublicGroupChatSearchManager.h"
+#include "PublicGroupChatSearchController.h"
 #include "QmlUtils.h"
+#include "RegistrationController.h"
 #include "RegistrationDataFormFilterModel.h"
-#include "RegistrationManager.h"
-#include "RosterFilterProxyModel.h"
+#include "RosterController.h"
+#include "RosterFilterModel.h"
 #include "RosterItemWatcher.h"
-#include "RosterManager.h"
 #include "RosterModel.h"
-#include "ServerFeaturesCache.h"
+#include "Settings.h"
 #include "StatusBar.h"
 #include "TextFormatter.h"
 #include "UserDevicesModel.h"
-#include "VCardManager.h"
+#include "VCardController.h"
 #include "VCardModel.h"
-#include "VersionManager.h"
+#include "VersionController.h"
 
 Q_DECLARE_ASSOCIATIVE_CONTAINER_METATYPE(QMultiHash)
 
@@ -141,15 +145,22 @@ Q_DECLARE_METATYPE(std::function<void(GroupChatUser &)>)
 
 Q_DECLARE_METATYPE(std::shared_ptr<Message>)
 
-#ifndef QAPPLICATION_CLASS
-#define QAPPLICATION_CLASS QApplication
-#endif
-#include QT_STRINGIFY(QAPPLICATION_CLASS)
-
 const auto QUICK_CONTROLS_STYLE_VARIABLE = "QT_QUICK_CONTROLS_STYLE";
 const auto QUICK_CONTROLS_DEFAULT_DESKTOP_STYLE = QStringLiteral("org.kde.desktop");
 const auto QUICK_CONTROLS_DEFAULT_MOBILE_STYLE = QStringLiteral("Material");
 
+// Environment variable containing strings used to filter out log messages that contain them.
+// The strings must be separated by semicolons.
+const auto KAIDAN_LOG_FILTER_VARIABLE = "KAIDAN_LOG_FILTER";
+constexpr auto KAIDAN_LOG_FILTER_SEPARATOR = QLatin1Char(';');
+// Strings used to always filter out log messages containing them.
+const QStringList KAIDAN_DEFAULT_FILTER_STRINGS = {
+    QStringLiteral("Previously registered enum will be overwritten due to name clash"),
+    QStringLiteral("Possible conflicting items"),
+    QStringLiteral("from scope * injected by *"),
+};
+QtMessageHandler originalLogMessageHandler = nullptr;
+
 enum CommandLineParseResult {
     CommandLineOk,
     CommandLineError,
@@ -165,7 +176,6 @@ CommandLineParseResult parseCommandLine(QCommandLineParser &parser, QString *err
     // add all possible arguments
     QCommandLineOption helpOption = parser.addHelpOption();
     QCommandLineOption versionOption = parser.addVersionOption();
-    parser.addOption({QStringLiteral("disable-xml-log"), QStringLiteral("Disable output of full XMPP XML stream.")});
 #ifndef NDEBUG
     parser.addOption({{QStringLiteral("m"), QStringLiteral("multiple")}, QStringLiteral("Allow multiple instances to be started.")});
 #endif
@@ -187,8 +197,44 @@ CommandLineParseResult parseCommandLine(QCommandLineParser &parser, QString *err
     return CommandLineOk;
 }
 
+void filterLog(QtMsgType type, const QMessageLogContext &context, const QString &msg)
+{
+    using namespace Qt::Literals::StringLiterals;
+    static QAtomicInt initialized;
+    static QMutex mutex;
+    static QStringList filterStrings;
+
+    if (initialized == 0) {
+        QMutexLocker locker(&mutex);
+
+        if (initialized == 0) {
+            filterStrings = qEnvironmentVariable(KAIDAN_LOG_FILTER_VARIABLE).split(KAIDAN_LOG_FILTER_SEPARATOR, Qt::SkipEmptyParts);
+            filterStrings.append(KAIDAN_DEFAULT_FILTER_STRINGS);
+
+            initialized = 1;
+        }
+    }
+
+    if (const auto match = std::find_if(filterStrings.cbegin(),
+                                        filterStrings.cend(),
+                                        [&msg](const QString &filter) {
+                                            const auto rx =
+                                                QRegularExpression::fromWildcard(filter, Qt::CaseInsensitive, QRegularExpression::UnanchoredWildcardConversion);
+                                            return msg.contains(rx);
+                                        });
+        match != filterStrings.cend()) {
+        return;
+    }
+
+    if (originalLogMessageHandler) {
+        (*originalLogMessageHandler)(type, context, msg);
+    }
+}
+
 Q_DECL_EXPORT int main(int argc, char *argv[])
 {
+    originalLogMessageHandler = qInstallMessageHandler(filterLog);
+
 #ifdef Q_OS_WIN
     if (AttachConsole(ATTACH_PARENT_PROCESS)) {
         freopen("CONOUT$", "w", stdout);
@@ -249,25 +295,25 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
         QIcon::setFallbackThemeName(QStringLiteral("breeze"));
     }
 
-    QAPPLICATION_CLASS app(argc, argv);
+    QApplication app(argc, argv);
 
     // register qMetaTypes
     qRegisterMetaType<ProviderListItem>();
     qRegisterMetaType<RosterItem>();
     qRegisterMetaType<RosterItemWatcher *>();
     qRegisterMetaType<RosterModel *>();
-    qRegisterMetaType<RosterManager *>();
+    qRegisterMetaType<RosterController *>();
     qRegisterMetaType<Message>();
     qRegisterMetaType<MessageModel *>();
     qRegisterMetaType<ChatHintModel *>();
     qRegisterMetaType<DiscoveryManager *>();
-    qRegisterMetaType<VCardManager *>();
-    qRegisterMetaType<VersionManager *>();
-    qRegisterMetaType<RegistrationManager *>();
-    qRegisterMetaType<AccountMigrationManager *>();
+    qRegisterMetaType<VCardController *>();
+    qRegisterMetaType<VersionController *>();
+    qRegisterMetaType<RegistrationController *>();
+    qRegisterMetaType<AccountMigrationController *>();
     qRegisterMetaType<FileSharingController *>();
     qRegisterMetaType<EncryptionController *>();
-    qRegisterMetaType<AvatarFileStorage *>();
+    qRegisterMetaType<AvatarCache *>();
     qRegisterMetaType<QmlUtils *>();
     qRegisterMetaType<QList<Message>>();
     qRegisterMetaType<QList<RosterItem>>();
@@ -282,7 +328,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<QXmppUri>();
     qRegisterMetaType<QMap<QString, QUrl>>();
     qRegisterMetaType<std::shared_ptr<Message>>();
-    qRegisterMetaType<AtmManager *>();
+    qRegisterMetaType<AtmController *>();
     qRegisterMetaType<GroupChatController *>();
     qRegisterMetaType<GroupChatUser>();
     qRegisterMetaType<QList<GroupChatUser>>();
@@ -310,12 +356,12 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qRegisterMetaType<MessageReactionDeliveryState>();
     qRegisterMetaType<FileModel::Role>();
     qRegisterMetaType<FileProxyModel::Mode>();
-    qRegisterMetaType<Account::ContactNotificationRule>();
-    qRegisterMetaType<Account::GroupChatNotificationRule>();
-    qRegisterMetaType<Account::AutomaticMediaDownloadsRule>();
-    qRegisterMetaType<Account::GeoLocationMapService>();
+    qRegisterMetaType<AccountSettings::PasswordVisibility>();
+    qRegisterMetaType<AccountSettings::AutomaticMediaDownloadsRule>();
+    qRegisterMetaType<AccountSettings::ContactNotificationRule>();
+    qRegisterMetaType<AccountSettings::GroupChatNotificationRule>();
+    qRegisterMetaType<AccountSettings::GeoLocationMapService>();
     qRegisterMetaType<RosterItem::AutomaticMediaDownloadsRule>();
-    qRegisterMetaType<AccountMigrationManager::MigrationState>();
     qRegisterMetaType<RosterItem::GroupChatFlag>();
     qRegisterMetaType<RosterModel::RosterItemRoles>();
     qRegisterMetaType<Message::TrustLevel>();
@@ -392,14 +438,12 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     }
 
 #endif // !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
-    //
-    // Kaidan back-end
-    //
-    Kaidan kaidan(!parser.isSet(QStringLiteral("disable-xml-log")));
+
+    MainController mainController;
 
 #if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
     // receive messages from other instances of Kaidan
-    QApplication::connect(&sapp, &KDSingleApplication::messageReceived, &app, [&kaidan](const QByteArray &messageData) {
+    QApplication::connect(&sapp, &KDSingleApplication::messageReceived, &app, [&mainController](const QByteArray &messageData) {
         QJsonDocument doc = QJsonDocument::fromJson(messageData);
         QJsonObject message = doc.object();
 
@@ -417,13 +461,13 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
             arguments << val.toString();
         }
 
-        kaidan.receiveMessage(arguments, message[QLatin1String("working_dir")].toString());
+        mainController.receiveMessage(arguments, message[QLatin1String("working_dir")].toString());
     });
 #endif // !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
 
     // open the XMPP-URI/link (if given)
     if (const auto positionalArguments = parser.positionalArguments(); !positionalArguments.isEmpty())
-        kaidan.addOpenUri(positionalArguments.first());
+        mainController.addOpenUri(positionalArguments.first());
 
     //
     // QML-GUI
@@ -445,10 +489,9 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<EmojiModel>(APPLICATION_ID, 1, 0, "EmojiModel");
     qmlRegisterType<EmojiProxyModel>(APPLICATION_ID, 1, 0, "EmojiProxyModel");
     qmlRegisterType<VCardModel>(APPLICATION_ID, 1, 0, "VCardModel");
-    qmlRegisterType<RosterFilterProxyModel>(APPLICATION_ID, 1, 0, "RosterFilterProxyModel");
+    qmlRegisterType<RosterFilterModel>(APPLICATION_ID, 1, 0, "RosterFilterModel");
     qmlRegisterType<BlockingModel>(APPLICATION_ID, 1, 0, "BlockingModel");
     qmlRegisterType<BlockingWatcher>(APPLICATION_ID, 1, 0, "BlockingWatcher");
-    qmlRegisterType<BlockingAction>(APPLICATION_ID, 1, 0, "BlockingAction");
     qmlRegisterType<ContactQrCodeGenerator>(APPLICATION_ID, 1, 0, "ContactQrCodeGenerator");
     qmlRegisterType<MessageComposition>(APPLICATION_ID, 1, 0, "MessageComposition");
     qmlRegisterType<FileSelectionModel>(APPLICATION_ID, 1, 0, "FileSelectionModel");
@@ -461,7 +504,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<FileProgressWatcher>(APPLICATION_ID, 1, 0, "FileProgressWatcher");
     qmlRegisterType<UserResourcesWatcher>(APPLICATION_ID, 1, 0, "UserResourcesWatcher");
     qmlRegisterType<RosterItemWatcher>(APPLICATION_ID, 1, 0, "RosterItemWatcher");
-    qmlRegisterType<PublicGroupChatSearchManager>(APPLICATION_ID, 1, 0, "PublicGroupChatSearchManager");
+    qmlRegisterType<PublicGroupChatSearchController>(APPLICATION_ID, 1, 0, "PublicGroupChatSearchController");
     qmlRegisterType<PublicGroupChatModel>(APPLICATION_ID, 1, 0, "PublicGroupChatModel");
     qmlRegisterType<PublicGroupChatProxyModel>(APPLICATION_ID, 1, 0, "PublicGroupChatProxyModel");
     qmlRegisterType<AuthenticatableEncryptionKeyModel>(APPLICATION_ID, 1, 0, "AuthenticatableEncryptionKeyModel");
@@ -473,47 +516,80 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterType<FileProxyModel>(APPLICATION_ID, 1, 0, "FileProxyModel");
     qmlRegisterType<TextFormatter>(APPLICATION_ID, 1, 0, "TextFormatter");
     qmlRegisterType<GroupChatQrCodeGenerator>(APPLICATION_ID, 1, 0, "GroupChatQrCodeGenerator");
+    qmlRegisterType<GroupChatInviteeFilterModel>(APPLICATION_ID, 1, 0, "GroupChatInviteeFilterModel");
     qmlRegisterType<GroupChatUserModel>(APPLICATION_ID, 1, 0, "GroupChatUserModel");
     qmlRegisterType<GroupChatUserFilterModel>(APPLICATION_ID, 1, 0, "GroupChatUserFilterModel");
     qmlRegisterType<GroupChatUserKeyAuthenticationFilterModel>(APPLICATION_ID, 1, 0, "GroupChatUserKeyAuthenticationFilterModel");
     qmlRegisterType<AccountTrustMessageUriGenerator>(APPLICATION_ID, 1, 0, "AccountTrustMessageUriGenerator");
     qmlRegisterType<ContactTrustMessageUriGenerator>(APPLICATION_ID, 1, 0, "ContactTrustMessageUriGenerator");
+    qmlRegisterType<ChatHintModel>(APPLICATION_ID, 1, 0, "ChatHintModel");
+    qmlRegisterType<MessageModel>(APPLICATION_ID, 1, 0, "MessageModel");
     qmlRegisterType<MessageReactionModel>(APPLICATION_ID, 1, 0, "MessageReactionModel");
+    qmlRegisterType<ChatController>(APPLICATION_ID, 1, 0, "ChatController");
+    qmlRegisterType<VersionController>(APPLICATION_ID, 1, 0, "VersionController");
+    qmlRegisterType<AvatarCache>(APPLICATION_ID, 1, 0, "AvatarCache");
+    qmlRegisterType<PresenceCache>(APPLICATION_ID, 1, 0, "PresenceCache");
 
+    // Q_OBJECT
     qmlRegisterUncreatableType<Account>(APPLICATION_ID, 1, 0, "Account", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<AccountSettings>(APPLICATION_ID, 1, 0, "AccountSettings", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<AtmController>(APPLICATION_ID, 1, 0, "AtmController", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<QAbstractItemModel>(APPLICATION_ID, 1, 0, "QAbstractItemModel", QStringLiteral("Used by proxy models"));
-    qmlRegisterUncreatableType<Emoji>(APPLICATION_ID, 1, 0, "Emoji", QStringLiteral("Used by emoji models"));
-    qmlRegisterUncreatableType<QMimeType>(APPLICATION_ID, 1, 0, "QMimeType", QStringLiteral("QMimeType type usable"));
     qmlRegisterUncreatableType<ClientWorker>(APPLICATION_ID, 1, 0, "ClientWorker", QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<DataFormModel>(APPLICATION_ID, 1, 0, "DataFormModel", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<Presence>(APPLICATION_ID, 1, 0, "Presence", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<RegistrationManager>(APPLICATION_ID, 1, 0, "RegistrationManager", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<AccountMigrationManager>(APPLICATION_ID,
-                                                        1,
-                                                        0,
-                                                        "AccountMigrationManager",
-                                                        QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<RegistrationController>(APPLICATION_ID,
+                                                       1,
+                                                       0,
+                                                       "RegistrationController",
+                                                       QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<AccountMigrationController>(APPLICATION_ID,
+                                                           1,
+                                                           0,
+                                                           "AccountMigrationController",
+                                                           QStringLiteral("Cannot create object; only enums defined!"));
     qmlRegisterUncreatableType<ChatState>(APPLICATION_ID, 1, 0, "ChatState", QStringLiteral("Cannot create object; only enums defined"));
     qmlRegisterUncreatableType<RosterModel>(APPLICATION_ID, 1, 0, "RosterModel", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<ServerFeaturesCache>(APPLICATION_ID, 1, 0, "ServerFeaturesCache", QStringLiteral("ServerFeaturesCache type usable"));
-    qmlRegisterUncreatableType<Encryption>(APPLICATION_ID, 1, 0, "Encryption", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<File>(APPLICATION_ID, 1, 0, "File", QStringLiteral("Not creatable from QML"));
-    qmlRegisterUncreatableType<PublicGroupChat>(APPLICATION_ID, 1, 0, "PublicGroupChat", QStringLiteral("Used by PublicGroupChatModel"));
     qmlRegisterUncreatableType<HostCompletionModel>(APPLICATION_ID, 1, 0, "HostCompletionModel", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<MessageReactionDeliveryState>(APPLICATION_ID,
-                                                             1,
-                                                             0,
-                                                             "MessageReactionDeliveryState",
-                                                             QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<RosterItem>(APPLICATION_ID, 1, 0, "RosterItem", QStringLiteral("Cannot create object; only enums defined!"));
-    qmlRegisterUncreatableType<Message>(APPLICATION_ID, 1, 0, "Message", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableType<RosterController>(APPLICATION_ID, 1, 0, "RosterController", QStringLiteral("Cannot create object; only enums defined!"));
 
+    // Q_GADGET
+    qmlRegisterUncreatableMetaObject(Emoji::staticMetaObject, APPLICATION_ID, 1, 0, "Emoji", QStringLiteral("Used by emoji models"));
+    qmlRegisterUncreatableMetaObject(Presence::staticMetaObject, APPLICATION_ID, 1, 0, "Presence", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableMetaObject(Encryption::staticMetaObject,
+                                     APPLICATION_ID,
+                                     1,
+                                     0,
+                                     "Encryption",
+                                     QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableMetaObject(File::staticMetaObject, APPLICATION_ID, 1, 0, "File", QStringLiteral("Not creatable from QML"));
+    qmlRegisterUncreatableMetaObject(PublicGroupChat::staticMetaObject,
+                                     APPLICATION_ID,
+                                     1,
+                                     0,
+                                     "PublicGroupChat",
+                                     QStringLiteral("Used by PublicGroupChatModel"));
+    qmlRegisterUncreatableMetaObject(MessageReactionDeliveryState::staticMetaObject,
+                                     APPLICATION_ID,
+                                     1,
+                                     0,
+                                     "MessageReactionDeliveryState",
+                                     QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableMetaObject(RosterItem::staticMetaObject,
+                                     APPLICATION_ID,
+                                     1,
+                                     0,
+                                     "RosterItem",
+                                     QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableMetaObject(Message::staticMetaObject, APPLICATION_ID, 1, 0, "Message", QStringLiteral("Cannot create object; only enums defined!"));
+    qmlRegisterUncreatableMetaObject(QMimeType::staticMetaObject, APPLICATION_ID, 1, 0, "QMimeType", QStringLiteral("QMimeType type usable"));
     qmlRegisterUncreatableMetaObject(ChatState::staticMetaObject,
                                      APPLICATION_ID,
                                      1,
                                      0,
                                      "ChatState",
                                      QStringLiteral("Can't create object; only enums defined!"));
+
+    // Q_NAMESPACE
     qmlRegisterUncreatableMetaObject(Enums::staticMetaObject, APPLICATION_ID, 1, 0, "Enums", QStringLiteral("Can't create object; only enums defined!"));
 
     qmlRegisterSingletonType<QmlUtils>(APPLICATION_ID, 1, 0, "Utils", [](QQmlEngine *, QJSEngine *) {
@@ -523,34 +599,22 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
         QObject *instance = new MediaUtils(qApp);
         return instance;
     });
-    qmlRegisterSingletonType<Kaidan>(APPLICATION_ID, 1, 0, "Kaidan", [](QQmlEngine *engine, QJSEngine *) {
-        engine->setObjectOwnership(Kaidan::instance(), QQmlEngine::CppOwnership);
-        return static_cast<QObject *>(Kaidan::instance());
+    qmlRegisterSingletonType<MainController>(APPLICATION_ID, 1, 0, "MainController", [](QQmlEngine *engine, QJSEngine *) {
+        engine->setObjectOwnership(MainController::instance(), QQmlEngine::CppOwnership);
+        return static_cast<QObject *>(MainController::instance());
     });
     qmlRegisterSingletonType<GuiStyle>(APPLICATION_ID, 1, 0, "Style", [](QQmlEngine *, QJSEngine *) {
         return static_cast<QObject *>(new GuiStyle(QCoreApplication::instance()));
     });
-    qmlRegisterSingletonType<AccountManager>(APPLICATION_ID, 1, 0, "AccountManager", [](QQmlEngine *, QJSEngine *) {
-        return static_cast<QObject *>(AccountManager::instance());
+    qmlRegisterSingletonType<Settings>(APPLICATION_ID, 1, 0, "Settings", [](QQmlEngine *, QJSEngine *) {
+        return static_cast<QObject *>(Settings::instance());
     });
-    qmlRegisterSingletonType<ChatController>(APPLICATION_ID, 1, 0, "ChatController", [](QQmlEngine *, QJSEngine *) {
-        return static_cast<QObject *>(ChatController::instance());
-    });
-    qmlRegisterSingletonType<GroupChatController>(APPLICATION_ID, 1, 0, "GroupChatController", [](QQmlEngine *, QJSEngine *) {
-        return static_cast<QObject *>(GroupChatController::instance());
-    });
-    qmlRegisterSingletonType<EncryptionController>(APPLICATION_ID, 1, 0, "EncryptionController", [](QQmlEngine *, QJSEngine *) {
-        return static_cast<QObject *>(EncryptionController::instance());
+    qmlRegisterSingletonType<AccountController>(APPLICATION_ID, 1, 0, "AccountController", [](QQmlEngine *, QJSEngine *) {
+        return static_cast<QObject *>(AccountController::instance());
     });
     qmlRegisterSingletonType<RosterModel>(APPLICATION_ID, 1, 0, "RosterModel", [](QQmlEngine *, QJSEngine *) {
         return static_cast<QObject *>(RosterModel::instance());
     });
-    qmlRegisterSingletonType<MessageModel>(APPLICATION_ID, 1, 0, "MessageModel", [](QQmlEngine *, QJSEngine *) {
-        return static_cast<QObject *>(MessageModel::instance());
-    });
-    qmlRegisterSingletonType<ChatHintModel>(APPLICATION_ID, 1, 0, "ChatHintModel", [](QQmlEngine *, QJSEngine *) {
-        return static_cast<QObject *>(ChatHintModel::instance());
-    });
     qmlRegisterSingletonType<HostCompletionModel>(APPLICATION_ID, 1, 0, "HostCompletionModel", [](QQmlEngine *, QJSEngine *) {
         static auto self = new HostCompletionModel(qApp);
         return static_cast<QObject *>(self);
