@@ -2,23 +2,28 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+// Qt
 #include <QFile>
 #include <QImage>
 #include <QMimeDatabase>
 #include <QMimeType>
+#include <QSignalSpy>
 #include <QTemporaryDir>
 #include <QTest>
+// Kaidan
+#include "FileModel.h"
+#include "FileProxyModel.h"
+#include "Test.h"
 
-#include "../src/FileModel.h"
-#include "../src/FileProxyModel.h"
-
-class FileModelTest : public QObject
+class FileModelTest : public Test
 {
     Q_OBJECT
 
 private Q_SLOTS:
-    void initTestCase()
+    void initTestCase() override
     {
+        Test::initTestCase();
+
         qRegisterMetaType<QPersistentModelIndex>();
         qRegisterMetaType<QList<QPersistentModelIndex>>();
         qRegisterMetaType<QAbstractItemModel::LayoutChangeHint>();
@@ -35,7 +40,7 @@ private Q_SLOTS:
         QVERIFY(!fileInfo.exists());
     }
 
-    void test_File_Proxy_Model()
+    void testFileProxyModel()
     {
         FileModel model;
         FileProxyModel proxy;
@@ -179,7 +184,7 @@ private:
 
     QString filePath(const QString &content) const
     {
-        return m_dir.filePath(QStringLiteral("%1.txt").arg(qChecksum(content.toUtf8().constData(), content.length())));
+        return m_dir.filePath(QStringLiteral("%1.txt").arg(qChecksum(QByteArrayView(content.toUtf8().constData(), content.length()))));
     }
 
     QImage createImage(const QColor &color) const
@@ -229,7 +234,7 @@ private:
         const QFileInfo fileInfo(filePath(color));
         File file;
 
-        file.id = qChecksum(fileInfo.fileName().toUtf8().constData(), fileInfo.fileName().length());
+        file.id = qChecksum(QByteArrayView(fileInfo.fileName().toUtf8().constData(), fileInfo.fileName().length()));
         file.fileGroupId = file.id - 10;
         file.name = fileInfo.baseName();
         file.mimeType = m_db.mimeTypeForFile(fileInfo);
@@ -250,7 +255,7 @@ private:
         const QFileInfo fileInfo(filePath(content));
         File file;
 
-        file.id = qChecksum(fileInfo.fileName().toUtf8().constData(), fileInfo.fileName().length());
+        file.id = qChecksum(QByteArrayView(fileInfo.fileName().toUtf8().constData(), fileInfo.fileName().length()));
         file.fileGroupId = file.id - 10;
         file.name = fileInfo.baseName();
         file.mimeType = m_db.mimeTypeForFile(fileInfo);
