@@ -2,10 +2,13 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-#include "../src/FutureUtils.h"
+// Qt
 #include <QTest>
+// Kaidan
+#include "FutureUtils.h"
+#include "Test.h"
 
-class FutureUtilsTest : public QObject
+class FutureUtilsTest : public Test
 {
     Q_OBJECT
 
