@@ -2,20 +2,23 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+// Qt
+#include <QSignalSpy>
 #include <QTest>
-
+// QXmpp
 #include <QXmppUtils.h>
-
-#include "../src/PresenceCache.h"
+// Kaidan
+#include "PresenceCache.h"
+#include "Test.h"
 
 Q_DECLARE_METATYPE(QList<QXmppPresence>)
 
-class PresenceCacheTest : public QObject
+class PresenceCacheTest : public Test
 {
     Q_OBJECT
 
 private:
-    Q_SLOT void initTestCase();
+    Q_SLOT void initTestCase() override;
     Q_SLOT void presenceChangedSignal();
     Q_SLOT void presenceGetter_data();
     Q_SLOT void presenceGetter();
@@ -37,6 +40,8 @@ private:
 
 void PresenceCacheTest::initTestCase()
 {
+    Test::initTestCase();
+
     qRegisterMetaType<PresenceCache::ChangeType>();
 }
 
@@ -51,11 +56,11 @@ void PresenceCacheTest::presenceChangedSignal()
     };
 
     QXmppPresence p1;
-    p1.setFrom("bob@kaidan.im/dev1");
+    p1.setFrom(QStringLiteral("bob@kaidan.im/dev1"));
     QXmppPresence p2(QXmppPresence::Unavailable);
-    p2.setFrom("bob@kaidan.im/dev1");
+    p2.setFrom(QStringLiteral("bob@kaidan.im/dev1"));
     QXmppPresence p3(QXmppPresence::Unavailable);
-    p3.setFrom("bob@kaidan.im/dev23");
+    p3.setFrom(QStringLiteral("bob@kaidan.im/dev23"));
 
     QSignalSpy spy(&cache, &PresenceCache::presenceChanged);
     auto signalCount = 0;
@@ -66,8 +71,8 @@ void PresenceCacheTest::presenceChangedSignal()
     cache.updatePresence(p1);
     QCOMPARE(spy.count(), ++signalCount);
     QCOMPARE(spy[0][ChangeType], QVariant::fromValue(PresenceCache::Connected));
-    QCOMPARE(spy[0][Jid], "bob@kaidan.im");
-    QCOMPARE(spy[0][Resource], "dev1");
+    QCOMPARE(spy[0][Jid], QStringLiteral("bob@kaidan.im"));
+    QCOMPARE(spy[0][Resource], QStringLiteral("dev1"));
 
     cache.updatePresence(p1);
     QCOMPARE(spy.count(), ++signalCount);
@@ -92,11 +97,11 @@ void PresenceCacheTest::presenceGetter_data()
     QTest::addColumn<QString>("resource");
     QTest::addColumn<bool>("exists");
 
-    QTest::newRow("bob1") << "bob@kaidan.im" << "dev1" << true;
-    QTest::newRow("bob2") << "bob@kaidan.im" << "dev2" << true;
-    QTest::newRow("alice1") << "alice@kaidan.im" << "kdn1" << true;
-    QTest::newRow("alice2") << "alice@kaidan.im" << "kdn2" << true;
-    QTest::newRow("unknwon presence") << "alice@kaidan.im" << "device3" << false;
+    QTest::newRow("bob1") << QStringLiteral("bob@kaidan.im") << QStringLiteral("dev1") << true;
+    QTest::newRow("bob2") << QStringLiteral("bob@kaidan.im") << QStringLiteral("dev2") << true;
+    QTest::newRow("alice1") << QStringLiteral("alice@kaidan.im") << QStringLiteral("kdn1") << true;
+    QTest::newRow("alice2") << QStringLiteral("alice@kaidan.im") << QStringLiteral("kdn2") << true;
+    QTest::newRow("unknwon presence") << QStringLiteral("alice@kaidan.im") << QStringLiteral("device3") << false;
 }
 
 void PresenceCacheTest::presenceGetter()
@@ -123,25 +128,33 @@ void PresenceCacheTest::idealResource_data()
 #define ROW(name, presences, expectedResource) QTest::newRow(name) << (presences) << QStringLiteral(expectedResource)
 
     ROW("DND",
-        QList<QXmppPresence>() << simplePresence("a@b/1", QXmppPresence::Away) << simplePresence("a@b/2", QXmppPresence::XA)
-                               << simplePresence("a@b/3", QXmppPresence::Online) << simplePresence("a@b/4", QXmppPresence::Chat)
-                               << simplePresence("a@b/5", QXmppPresence::DND),
+        QList<QXmppPresence>() << simplePresence(QStringLiteral("a@b/1"), QXmppPresence::Away) << simplePresence(QStringLiteral("a@b/2"), QXmppPresence::XA)
+                               << simplePresence(QStringLiteral("a@b/3"), QXmppPresence::Online) << simplePresence(QStringLiteral("a@b/4"), QXmppPresence::Chat)
+                               << simplePresence(QStringLiteral("a@b/5"), QXmppPresence::DND),
         "5");
     ROW("Chat",
-        QList<QXmppPresence>() << simplePresence("a@b/1", QXmppPresence::Away) << simplePresence("a@b/2", QXmppPresence::XA)
-                               << simplePresence("a@b/3", QXmppPresence::Chat) << simplePresence("a@b/4", QXmppPresence::Online),
+        QList<QXmppPresence>() << simplePresence(QStringLiteral("a@b/1"), QXmppPresence::Away) << simplePresence(QStringLiteral("a@b/2"), QXmppPresence::XA)
+                               << simplePresence(QStringLiteral("a@b/3"), QXmppPresence::Chat)
+                               << simplePresence(QStringLiteral("a@b/4"), QXmppPresence::Online),
         "3");
-    ROW("XA", QList<QXmppPresence>() << simplePresence("a@b/1", QXmppPresence::XA) << simplePresence("a@b/2", QXmppPresence::Away), "1");
-    ROW("Away", QList<QXmppPresence>() << simplePresence("a@b/1", QXmppPresence::Away) << simplePresence("a@b/2", QXmppPresence::XA), "1");
+    ROW("XA",
+        QList<QXmppPresence>() << simplePresence(QStringLiteral("a@b/1"), QXmppPresence::XA) << simplePresence(QStringLiteral("a@b/2"), QXmppPresence::Away),
+        "1");
+    ROW("Away",
+        QList<QXmppPresence>() << simplePresence(QStringLiteral("a@b/1"), QXmppPresence::Away) << simplePresence(QStringLiteral("a@b/2"), QXmppPresence::XA),
+        "1");
     ROW("status text [DND]",
-        QList<QXmppPresence>() << simplePresence("a@b/1", QXmppPresence::Away) << simplePresence("a@b/2", QXmppPresence::XA)
-                               << simplePresence("a@b/3", QXmppPresence::Online) << simplePresence("a@b/4", QXmppPresence::Chat)
-                               << simplePresence("a@b/5", QXmppPresence::DND) << simplePresence("a@b/6", QXmppPresence::Chat, "my status")
-                               << simplePresence("a@b/7", QXmppPresence::DND, "my status") << simplePresence("a@b/8", QXmppPresence::Away, "my status"),
+        QList<QXmppPresence>() << simplePresence(QStringLiteral("a@b/1"), QXmppPresence::Away) << simplePresence(QStringLiteral("a@b/2"), QXmppPresence::XA)
+                               << simplePresence(QStringLiteral("a@b/3"), QXmppPresence::Online) << simplePresence(QStringLiteral("a@b/4"), QXmppPresence::Chat)
+                               << simplePresence(QStringLiteral("a@b/5"), QXmppPresence::DND)
+                               << simplePresence(QStringLiteral("a@b/6"), QXmppPresence::Chat, QStringLiteral("my status"))
+                               << simplePresence(QStringLiteral("a@b/7"), QXmppPresence::DND, QStringLiteral("my status"))
+                               << simplePresence(QStringLiteral("a@b/8"), QXmppPresence::Away, QStringLiteral("my status")),
         "7");
     ROW("status text [XA]",
-        QList<QXmppPresence>() << simplePresence("a@b/1", QXmppPresence::Away) << simplePresence("a@b/2", QXmppPresence::XA)
-                               << simplePresence("a@b/3", QXmppPresence::XA, "my status") << simplePresence("a@b/4", QXmppPresence::Away, "my status"),
+        QList<QXmppPresence>() << simplePresence(QStringLiteral("a@b/1"), QXmppPresence::Away) << simplePresence(QStringLiteral("a@b/2"), QXmppPresence::XA)
+                               << simplePresence(QStringLiteral("a@b/3"), QXmppPresence::XA, QStringLiteral("my status"))
+                               << simplePresence(QStringLiteral("a@b/4"), QXmppPresence::Away, QStringLiteral("my status")),
         "3");
 
 #undef ROW
@@ -162,10 +175,10 @@ void PresenceCacheTest::idealResource()
 
 void PresenceCacheTest::addBasicPresences()
 {
-    addSimplePresence("bob@kaidan.im/dev1");
-    addSimplePresence("bob@kaidan.im/dev2");
-    addSimplePresence("alice@kaidan.im/kdn1");
-    addSimplePresence("alice@kaidan.im/kdn2");
+    addSimplePresence(QStringLiteral("bob@kaidan.im/dev1"));
+    addSimplePresence(QStringLiteral("bob@kaidan.im/dev2"));
+    addSimplePresence(QStringLiteral("alice@kaidan.im/kdn1"));
+    addSimplePresence(QStringLiteral("alice@kaidan.im/kdn2"));
 }
 
 void PresenceCacheTest::addSimplePresence(const QString &jid, QXmppPresence::AvailableStatusType available, const QString &status, QXmppPresence::Type type)
